﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32;

namespace ColorSelectorAddIn
{

    public partial class PaletteControl : UserControl
    {
        #region Fields

        string _code = "";
        bool _generateVB = false;

        #endregion

        #region Properties

        public string Code
        {
            get { return _code; }
        }

        public bool GenerateVB
        {
            get { return _generateVB; }
        }

        #endregion

        #region Ctor(s)

        public PaletteControl()
        {
            InitializeComponent();
            this.pictureBoxPalette.MouseMove += new MouseEventHandler(pictureBoxPalette_MouseMove);
            this.pictureBoxPalette.Cursor = System.Windows.Forms.Cursors.Cross;
        }

        #endregion

        #region Event Handlers and Delegates

        void pictureBoxPalette_MouseMove(object sender, MouseEventArgs e)
        {
            // Get the color under the current pointer position
            Color color = GetPointColor(e.X, e.Y);

            // Update the RGB labels and the 2nd pic box
            // using the retrieved color
            DisplayColor(color);

            // Generate our VB or C# code for the Color
            // structure
            SetCode(color, _generateVB);
        }

        public event EventHandler ColorSelected;

        protected virtual void OnColorSelected(EventArgs e)
        {
            if (ColorSelected != null)
                ColorSelected(this, e);
        }

        private void pictureBoxPalette_Click(object sender, EventArgs e)
        {
            OnColorSelected(new EventArgs());
        }

        #endregion

        #region Private Routines

        /// <summary>
        /// Returns a Color structure representing the color of
        /// the pixel at the indicated x and y coordinates.
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <returns>A Color structure</returns>
        private Color GetPointColor(int x, int y)
        {
            // Get the bitmap from the palette picture box
            Bitmap bmp = (Bitmap)pictureBoxPalette.Image;
            // Use GetPixel to retrieve a color
            // structure for the current pointer position
            Color color = bmp.GetPixel(x, y);

            // Return the color structure
            return color;
        }

        /// <summary>
        /// Displays the RGB values for the given color. Also sets
        /// the background color of the secondary picture box.
        /// </summary>
        /// <param name="color">The Color to display</param>
        private void DisplayColor(Color color)
        {
            // pull out the RGB values from the
            // color structure
            string R = color.R.ToString();
            string G = color.G.ToString();
            string B = color.B.ToString();

            // set our secondary picture box
            // to display the current color
            this.pictureBoxColor.BackColor = color;

            // display RGB values in the label
            // controls
            this.labelRValue.Text = R;
            this.labelGValue.Text = G;
            this.labelBValue.Text = B;
        }

        /// <summary>
        /// Generates a string representing the C# or VB code necessary to
        /// create a Color structure instance that matches the passed in
        /// Color structure. This string is then assigned to this
        /// user control's _code field.
        /// </summary>
        /// <param name="color">The color to represent in code.</param>
        /// <param name="isVB">Boolean flag indicating the language
        /// to use: false indicates C#, true indicates VB</param>
        private void SetCode(Color color, bool isVB)
        {
            // Read in add-in settings from registry
            SetPropFromReg();

            string code = "";

            if (isVB)
            {
                code = "Dim color As Color = ";
            }
            else
            {
                code = "Color color = ";
            }

            code = code + "Color.FromArgb(" + color.R.ToString() + ", " +
                color.G.ToString() + ", " +
                color.B.ToString() + ");";

            _code = code;
            this.labelCode.Text = _code;

        }
        /// <summary>
        /// Reads a registry entry and sets the language output fields
        /// appropriately.
        /// </summary>
        private void SetPropFromReg()
        {
            RegistryKey regKey =
              Registry.CurrentUser.OpenSubKey(@"Software\Contoso\Addins\ColorPalette");
            string codeVal = (string)regKey.GetValue("Language", "CSharp");

            if (codeVal == "CSharp")
            {
                _generateVB = false;
            }
            else
            {
                _generateVB = true;
            }
        }
        #endregion



    }
}
